import { act, render, screen } from '@testing-library/react';
import { StoreType } from '@/redux/store';
import { bioEntitiesContentFixture } from '@/models/fixtures/bioEntityContentsFixture';
import {
  InitialStoreState,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import { drawerSearchStepOneFixture } from '@/redux/drawer/drawerFixture';
import { BioEntitiesSubmapItem } from './BioEntitiesSubmapItem.component';

const SECOND_STEP = 2;

const renderComponent = (initialStoreState: InitialStoreState = {}): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <BioEntitiesSubmapItem
          mapName="main map"
          numberOfEntities={21}
          bioEntities={bioEntitiesContentFixture}
        />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('BioEntitiesSubmapItem - component', () => {
  it('should display map name, number of elements, icon', () => {
    renderComponent();

    expect(screen.getByText('main map (21)')).toBeInTheDocument();
    expect(screen.getByTestId('arrow-icon')).toBeInTheDocument();
  });
  it('should navigate user to bio enitites results list after clicking button', async () => {
    const { store } = renderComponent({
      bioEntity: {
        data: [
          {
            searchQueryElement: '',
            loading: 'succeeded',
            error: { name: '', message: '' },
            data: bioEntitiesContentFixture,
          },
        ],
        loading: 'succeeded',
        error: { name: '', message: '' },
      },
      drawer: drawerSearchStepOneFixture,
    });

    const navigationButton = screen.getByTestId('bio-entites-submap-button');
    await act(() => {
      navigationButton.click();
    });

    const {
      drawer: {
        searchDrawerState: { stepType, selectedValue, currentStep, listOfBioEnitites },
      },
    } = store.getState();

    expect(stepType).toBe('bioEntity');
    expect(selectedValue).toBe(undefined);
    expect(currentStep).toBe(SECOND_STEP);
    expect(listOfBioEnitites).toBe(bioEntitiesContentFixture);
  });
});
