/* eslint-disable no-magic-numbers */
import { render, screen } from '@testing-library/react';
import {
  InitialStoreState,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import { drugsFixture } from '@/models/fixtures/drugFixtures';
import { StoreType } from '@/redux/store';
import { PinDetailsItem } from '@/types/models';
import { PinType } from '../PinsList.types';
import { PinsListItem } from './PinsListItem.component';

const DRUGS_PIN = {
  name: drugsFixture[0].targets[0].name,
  pin: drugsFixture[0].targets[0],
};

const renderComponent = (
  name: string,
  pin: PinDetailsItem,
  type: PinType,
  initialStoreState: InitialStoreState = {},
): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <PinsListItem name={name} type={type} pin={pin} />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('PinsListItem - component ', () => {
  it('should display full name of pin', () => {
    renderComponent(DRUGS_PIN.name, DRUGS_PIN.pin, 'drugs');

    const drugName = drugsFixture[0].targets[0].name;

    expect(screen.getByText(drugName)).toBeInTheDocument();
  });
  it('should display list of elements for pin', () => {
    renderComponent(DRUGS_PIN.name, DRUGS_PIN.pin, 'drugs');

    const firstPinElementType = drugsFixture[0].targets[0].targetParticipants[0].type;
    const firstPinElementResource = drugsFixture[0].targets[0].targetParticipants[0].resource;
    const secondPinElementType = drugsFixture[0].targets[0].targetParticipants[1].type;
    const secondPinElementResource = drugsFixture[0].targets[0].targetParticipants[1].resource;

    expect(screen.getByText(firstPinElementType, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(firstPinElementResource, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(secondPinElementType, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(secondPinElementResource, { exact: false })).toBeInTheDocument();
  });
  it('should display list of references for pin', () => {
    renderComponent(DRUGS_PIN.name, DRUGS_PIN.pin, 'drugs');

    const firstPinReferenceType = drugsFixture[0].targets[0].references[0].type;
    const firstPinReferenceResource = drugsFixture[0].targets[0].references[0].resource;
    const secondPinReferenceType = drugsFixture[0].targets[0].references[1].type;
    const secondPinReferenceResource = drugsFixture[0].targets[0].references[1].resource;

    expect(screen.getByText(firstPinReferenceType, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(firstPinReferenceResource, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(secondPinReferenceType, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(secondPinReferenceResource, { exact: false })).toBeInTheDocument();
  });
});
