/* eslint-disable no-magic-numbers */
import { PIN_SIZE } from '@/constants/canvas';
import { allBioEntitesSelectorOfCurrentMap } from '@/redux/bioEntity/bioEntity.selectors';
import { BioEntity } from '@/types/models';
import { UsePointToProjectionResult, usePointToProjection } from '@/utils/map/usePointToProjection';
import { Feature } from 'ol';
import { Point as OlPoint } from 'ol/geom';
import BaseLayer from 'ol/layer/Base';
import VectorLayer from 'ol/layer/Vector';
import VectorSource from 'ol/source/Vector';
import Icon from 'ol/style/Icon';
import Style from 'ol/style/Style';
import { useMemo } from 'react';
import { useSelector } from 'react-redux';
import { getCanvasIcon } from './getCanvasIcon';

const getPinFeature = (
  { x, y, width, height, name }: BioEntity,
  pointToProjection: UsePointToProjectionResult,
): Feature => {
  const point = {
    x: x + width / 2,
    y: y + height / 2,
  };

  return new Feature({
    geometry: new OlPoint(pointToProjection(point)),
    name,
  });
};

const getPinStyle = ({ value, color }: { value: number; color: string }): Style =>
  new Style({
    image: new Icon({
      displacement: [0, PIN_SIZE.height],
      anchorXUnits: 'fraction',
      anchorYUnits: 'pixels',
      img: getCanvasIcon({
        color,
        value,
      }),
    }),
  });

export const useOlMapPinsLayer = (): BaseLayer => {
  const pointToProjection = usePointToProjection();
  const bioEntites = useSelector(allBioEntitesSelectorOfCurrentMap);

  const bioEntityFeatures = useMemo(
    () =>
      bioEntites.map(({ bioEntity }, index) => {
        const feature = getPinFeature(bioEntity, pointToProjection);
        const style = getPinStyle({
          color: '#106AD7',
          value: index + 1,
        });

        feature.setStyle(style);
        return feature;
      }),
    [bioEntites, pointToProjection],
  );

  const vectorSource = useMemo(() => {
    return new VectorSource({
      features: [...bioEntityFeatures],
    });
  }, [bioEntityFeatures]);

  const pinsLayer = useMemo(
    () =>
      new VectorLayer({
        source: vectorSource,
      }),
    [vectorSource],
  );

  return pinsLayer;
};
