import { SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { getMultiBioEntity } from '@/redux/bioEntity/bioEntity.thunks';
import { MapSize } from '@/redux/map/map.types';
import { AppDispatch } from '@/redux/store';
import { ElementSearchResult, Reaction } from '@/types/models';
import { latLngToPoint } from '@/utils/map/latLngToPoint';
import { getElementsByPoint } from '@/utils/search/getElementsByCoordinates';
import { PayloadAction } from '@reduxjs/toolkit';
import { MapBrowserEvent } from 'ol';
import { toLonLat } from 'ol/proj';
import { getReactionsByIds } from '../../../../../redux/reactions/reactions.thunks';

const FIRST = 0;

/* prettier-ignore */
export const handleAliasResults =
  (dispatch: AppDispatch) =>
    async ({ id }: ElementSearchResult): Promise<void> => {
      dispatch(
        getMultiBioEntity({
          searchQueries: [id.toString()],
          isPerfectMatch: true
        }),
      );
    };

/* prettier-ignore */
export const handleReactionResults =
  (dispatch: AppDispatch) =>
    async ({ id }: ElementSearchResult): Promise<void> => {
      const data = await dispatch(getReactionsByIds([id])) as PayloadAction<Reaction[] | undefined>;
      const payload = data?.payload;
      if (!data || !payload || payload.length === SIZE_OF_EMPTY_ARRAY) {
        return;
      }

      const { products, reactants, modifiers } = payload[FIRST];
      const productsIds = products.map(p => p.aliasId);
      const reactantsIds = reactants.map(r => r.aliasId);
      const modifiersIds = modifiers.map(m => m.aliasId);
      const bioEntitiesIds = [...productsIds, ...reactantsIds, ...modifiersIds].map(identifier => String(identifier));

      await dispatch(
        getMultiBioEntity({
          searchQueries: bioEntitiesIds,
          isPerfectMatch: true },
        ),
      );
    };

/* prettier-ignore */
export const onMapSingleClick =
  (mapSize: MapSize, modelId: number, dispatch: AppDispatch) =>
    async (e: MapBrowserEvent<UIEvent>): Promise<void> => {
      const [lng, lat] = toLonLat(e.coordinate);
      const point = latLngToPoint([lat, lng], mapSize);
      const searchResults = await getElementsByPoint({ point, currentModelId: modelId });

      if (!searchResults || searchResults.length === SIZE_OF_EMPTY_ARRAY) {
        return;
      }

      const closestSearchResult = searchResults[FIRST];
      const { type } = closestSearchResult;
      const action = {
        'ALIAS': handleAliasResults,
        'REACTION': handleReactionResults,
      }[type];

      await action(dispatch)(closestSearchResult);
    };
